import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository, Between, MoreThanOrEqual, LessThanOrEqual } from 'typeorm';
import { SystemUsage } from './system-usage.entity';
import { subMinutes, subHours, subDays } from 'date-fns';

@Injectable()
export class SystemUsageService {
  constructor(
    @InjectRepository(SystemUsage)
    private readonly systemUsageRepository: Repository<SystemUsage>,
  ) {}

  async create(data: {
    cpuUsage: number;
    gpuUsage: number;
    memoryUsage: number;
  }) {
    const record = this.systemUsageRepository.create(data);
    return this.systemUsageRepository.save(record);
  }

  async findAll() {
    return this.systemUsageRepository.find();
  }

  async findOne(id: number) {
    return this.systemUsageRepository.findOne({ where: { id } });
  }

  async getLatestUsage(interval: 'minute' | 'hour' | 'day' = 'minute') {
    const now = new Date();
    let timeFunction: (date: Date, amount: number) => Date;
    let intervalMinutes: number;
    
    switch (interval) {
      case 'minute':
        timeFunction = subMinutes;
        intervalMinutes = 1;
        break;
      case 'hour':
        timeFunction = subHours;
        intervalMinutes = 60;
        break;
      case 'day':
        timeFunction = subDays;
        intervalMinutes = 1440;
        break;
      default:
        timeFunction = subMinutes;
        intervalMinutes = 1;
    }

    // 计算10个间隔点的时间
    const targetTimes = Array.from({length: 10}, (_, i) => 
      timeFunction(now, i * intervalMinutes)
    ).reverse();

    // 获取每个间隔点之后的第一条记录
    const results = await Promise.all(
      targetTimes.map(time => 
        this.systemUsageRepository.findOne({
          where: {
            createdAt: MoreThanOrEqual(time)
          },
          order: {
            createdAt: 'ASC'
          }
        })
      )
    );

    return results.filter(Boolean) as SystemUsage[];
  }
}